import { RimrafOptions } from '.';
export declare const rimrafWindows: (path: string, opt: RimrafOptions, state?: symbol) => Promise<void>;
export declare const rimrafWindowsSync: (path: string, opt: RimrafOptions, state?: symbol) => void;
