import { RimrafOptions } from '.';
export declare const rimrafNative: (path: string, opt: RimrafOptions) => Promise<void>;
export declare const rimrafNativeSync: (path: string, opt: RimrafOptions) => void;
