import { RimrafOptions } from '.';
export declare const rimrafPosix: (path: string, opt: RimrafOptions) => Promise<any>;
export declare const rimrafPosixSync: (path: string, opt: RimrafOptions) => any;
