"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_js_1 = require("./index.js");
exports.default = (opt = {}) => {
    (0, index_js_1.assertRimrafOptions)(opt);
    return opt;
};
//# sourceMappingURL=opt-arg.js.map